/*
 * $Header: /home/cvspublic/jakarta-tomcat/src/share/org/apache/tomcat/core/RequestAdapter.java,v 1.3 1999/10/29 23:40:46 costin Exp $
 * $Revision: 1.3 $
 * $Date: 1999/10/29 23:40:46 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 


package org.apache.tomcat.core;

import org.apache.tomcat.util.*;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/* XXX
   - add comments
   - more clean-up
   - make it minimal!
*/

/**
 * Low-level representation of a Request in a server adapter ( connector ).
 * 
 * @author James Duncan Davidson [duncan@eng.sun.com]
 * @author Jason Hunter [jch@eng.sun.com]
 * @author James Todd [gonzo@eng.sun.com]
 * @author Harish Prabandham
 * @author costin@eng.sun.com
 */

public interface RequestAdapter {

    // Required - basic request fields

    public String getScheme();
    
    public String getMethod();

    public String getRequestURI();

    public String getQueryString();

    public String getProtocol();
    
    // XXX temporary - to keep everything working
    public MimeHeaders getMimeHeaders();
    
    public  Enumeration getHeaderNames();

    public  String getHeader(String name);

    /** You can either provide an implementation of ServletInputStream or
	return BufferedServletInputStream( RequsetAdapter ), and implement
	doRead();
     */
    public  ServletInputStream getInputStream() throws IOException;

    /** Fill in the buffer. This method is probably easier to implement than
	previous.
	This method should only be called from SerlvetInputStream implementations.
	No need to implement it if your adapter implements ServletInputStream.
     */
    public  int doRead( byte b[], int off, int len ) throws IOException;

    // Required - connection info 
    public String getServerName();
    
    public int getServerPort();

    public String getRemoteAddr();
    
    public String getRemoteHost();


    // -------------------- "cooked" info --------------------
    // Hints = return null if you don't know,
    // and Tom will find the value. You can also use the static
    // methods in RequestImpl

    /** Return the cookies
     */
    public String[] getCookieHeaders();

    // server may have it pre-calculated - return null if
    // it doesn't
    public String getContextPath();

    // What's between context path and servlet name ( /servlet )
    // A smart server may use arbitrary prefixes and rewriting
    public String getServletPrefix();

    // Servlet name ( a smart server may use aliases and rewriting !!! )
    public String getServletName();

    // What's after Servlet name, before "?"
    public String getPathInfo();

    // Parameters - if the server can parse parameters faster
    // Note: there is a tricky requirement in Servlet API
    // regarding POST parameters ( you can't read the body until you are
    // asked for the first parameter ). It's easy if you use callbacks (
    // this method will be called when needed, but don't read until you are
    // asked for params )
    public Enumeration getParameterNames();

    public String[] getParameterValues(String name);

    // You probably know the session ID if you are in a distributed engine, or
    // if you use a non-standard session identifier ( to integrate with "legacy"
    // apps ?)
    public String getRequestedSessionId();

    // Authentication - carefull :-)
    public String getAuthType();

    public String getRemoteUser();

    // Headers and special Headers. You may want to use a "fast" route for those,
    // since it's almost sure will be needed ( for example get "other" headers in
    // a callback.). Again - if you don't know, return null.
    
    public String getCharacterEncoding();

    public int getContentLength();
    
    public String getContentType();

    public  void recycle(); 


    // XXX I hate this - but the only way to remove this method from the
    // inteface is to implement it on top of doRead(b[]).
    // Don't use this method if you can ( it is bad for performance !!)
    public int doRead() throws IOException;
    

}
